#include <stdio.h>
#include <string.h>

#include "vocaboli.h"
#include "items.h"
#include "estrazione.h"

static const char kStrNomeFileDiz[] = "italiano.txt";

enum { // codici d'errore
    kErrNoErr,
    kErrNullPtr,
    kErrNoMem,
    kErrNoDiz,
    kErrNoVocaboliEstratti,
    kErrTotErr // la quantita' degli errori in questo elenco
};

static const char **gDiz = NULL;
static long gQVocaboliDiz = -1; // -1: elenco parole del dizionario non valido
static const char **gVocaboli = NULL;
static long gQVocaboli = -1; // -1: elenco parole non valido


int CaricaVocabolario( void ) {
    gQVocaboliDiz = GetAllItemsFromFile( kStrNomeFileDiz, '\n', &gDiz );

    if( gQVocaboliDiz > 1 ) { // ALMENO due parole nel vocabolario, dai!
        return kErrNoErr;
    }
    else {
        return kErrNoDiz;
    }
}


void DismettiVocabolario( void ) {
    FreeStringsMemory( &gDiz );
    gQVocaboliDiz = -1; // -1: elenco parole del dizionario non valido
}


const char **VocaboliVocabolario( void ) {
    return gDiz;
}


long QVocaboliVocabolario( void ) {
    return gQVocaboliDiz;
}


int EstraiVocaboli( long qVocaboli ) {
    if( qVocaboli > 0 ) {
        const char **tmp = calloc( qVocaboli, sizeof(*tmp) );

        if( tmp ) {
            long i, e=-999, u = UltimoEstratto();

            for( i=0; i<qVocaboli; ++i ) {
                e = Estrai();
                e = u!=e ? e : Estrai();

                tmp[i] = gDiz[e];

                u = e;
            }

            DismettiVocaboliEstratti();
            gVocaboli = tmp;
            gQVocaboli = qVocaboli;

            return kErrNoErr;
        }
        else {
            return kErrNoMem;
        }
    }
    else {
        return kErrNoVocaboliEstratti;
    }
}


void DismettiVocaboliEstratti( void ) {
    free( gVocaboli );
    gQVocaboli = -1; // -1: elenco parole non valido
}


const char **VocaboliEstratti( void ) {
    return gVocaboli;
}


long QVocaboliEstratti( void ) {
    return gQVocaboli;
}


int CreaElencoVocaboliEstratti( const char **elenco ) {
    if( elenco ) {
        if( gQVocaboli > -1 ) {
            char *tmp = NULL;
            size_t i, lElenco = 1;

            for( i=0; i<gQVocaboli; ++i )
                lElenco += strlen( gVocaboli[i] ) + 2;

            tmp = malloc( lElenco*sizeof(*tmp) );

            if( tmp ) {
                char *p = tmp;

                for( i=0; i<gQVocaboli; ++i )
                    p += sprintf( p, "%s\r\n", gVocaboli[i] );
                *(p-2) = '\0';

                *elenco = tmp;
                return kErrNoErr;
            }
            else {
                return kErrNoMem;
            }
        }
        else {
            return kErrNoVocaboliEstratti;
        }
    }
    else {
        return kErrNullPtr;
    }
}


const char *DescrizioneErrore( int codice ) {
    static const char *strErr[] = {
        "", // caso non utilizzato ma da NON eliminare
        "Mi sono imbattuto in un puntatore nullo\n il che  piuttosto imbarazzante... ",
        "Memoria insufficiente. ",
        "Impossibile aprire il dizionario. ",
        "Non  stato estratto alcun vocabolo. "
    };

    if( codice>kErrNoErr && codice<kErrTotErr )
        return strErr[codice];
    else  return codice!=kErrNoErr ? "S' verificato un errore imprevisto. " : NULL;
}
